"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var dependency_graph_1 = require("dependency-graph");
var graphql_tag_1 = require("graphql-tag");
exports.propsType = function (convert) { return function (_a, options) {
    var name = _a.name, operationType = _a.operationType;
    var noNamespaces = (options.data.root.config || { noNamespaces: false }).noNamespaces;
    if (operationType === 'mutation') {
        return "\n            Partial<\n                " + convert(name) + "ReactApollo.MutateProps<\n                                        " + (noNamespaces ? convert(name) : '') + "Mutation, \n                                        " + (noNamespaces ? convert(name) : '') + "Variables\n                                        >\n                >\n        ";
    }
    else {
        return "\n            Partial<\n                ReactApollo.DataProps<\n                                        " + (noNamespaces ? convert(name) : '') + convert(operationType) + ", \n                                        " + (noNamespaces ? convert(name) : '') + "Variables\n                                    >\n                    >\n        ";
    }
}; };
exports.generateFragments = function (convert) { return function (fragments, options) {
    var cachedFragments = {};
    if (!fragments) {
        return '';
    }
    var graph = new dependency_graph_1.DepGraph({ circular: true });
    fragments.forEach(function (fragment) {
        graph.addNode(fragment.name, fragment);
    });
    fragments.forEach(function (fragment) {
        var depends = extractFragments(fragment.document);
        if (depends) {
            depends.forEach(function (name) {
                graph.addDependency(fragment.name, name);
            });
        }
    });
    return graph
        .overallOrder()
        .map(function (name) { return generateFragment(graph.getNodeData(name), options); })
        .join('\n');
    function generateFragment(fragment, options) {
        var cached = cachedFragments[fragment.name];
        if (!cached) {
            cachedFragments[fragment.name] = fragment;
            var config = options.data.root.config || {};
            var pascalCasedFragmentName = convert(fragment.name);
            // fooBar, FooBar and foo_bar may cause conflict due to the pascalCase.
            // Because all of them will have same namespace FooBar
            if (config.noNamespaces) {
                return "\n          export const " + pascalCasedFragmentName + "FragmentDoc = " + exports.gql(convert)(fragment, options) + ";\n        ";
            }
            else {
                return "\n          export namespace " + pascalCasedFragmentName + " {\n            export const FragmentDoc = " + exports.gql(convert)(fragment, options) + ";\n          }\n        ";
            }
        }
        else {
            if (fragment.document !== cached.document) {
                throw new Error("Duplicated fragment called '" + fragment.name + "'");
            }
        }
    }
}; };
exports.gql = function (convert) { return function (operation, options) {
    var config = options.data.root.config || {};
    var doc = "\n    " + operation.document + "\n    " + includeFragments(transformFragments(convert)(operation.document, options)) + "\n  ";
    return config.noGraphqlTag ? JSON.stringify(graphql_tag_1.default(doc)) : 'gql`' + doc + '`';
}; };
function includeFragments(fragments) {
    if (fragments) {
        return "\n      " + fragments
            .filter(function (name, i, all) { return all.indexOf(name) === i; })
            .map(function (name) { return '${' + name + '}'; })
            .join('\n') + "\n    ";
    }
    return '';
}
function extractFragments(document) {
    return (document.match(/\.\.\.[a-z0-9\_]+/gi) || []).map(function (name) { return name.replace('...', ''); });
}
exports.extractFragments = extractFragments;
var transformFragments = function (convert) { return function (document, options) {
    return extractFragments(document).map(function (document) { return exports.toFragmentName(convert)(document, options); });
}; };
exports.toFragmentName = function (convert) { return function (fragmentName, options) {
    var config = options.data.root.config || {};
    if (config.noNamespaces) {
        return convert(fragmentName + "FragmentDoc");
    }
    else {
        return convert(fragmentName) + '.FragmentDoc';
    }
}; };
//# sourceMappingURL=helpers.js.map